package de.szut.casino.lootboxes;

import de.szut.casino.user.UserEntity;
import de.szut.casino.user.UserRepository;
import de.szut.casino.user.UserService;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;

@RestController
public class LootBoxController {
    private final LootBoxRepository lootBoxRepository;
    private final UserRepository userRepository;
    private final UserService userService;

    public LootBoxController(LootBoxRepository lootBoxRepository, UserRepository userRepository, UserService userService) {
        this.lootBoxRepository = lootBoxRepository;
        this.userRepository = userRepository;
        this.userService = userService;
    }

    @GetMapping("/lootboxes")
    public List<LootBoxEntity> getAllLootBoxes() {
        return lootBoxRepository.findAll();
    }

    @PostMapping("/lootboxes/{id}")
    public ResponseEntity<Object> purchaseLootBox(@PathVariable Long id, @RequestHeader("Authorization") String token) {
        Optional<LootBoxEntity> optionalLootBox = lootBoxRepository.findById(id);
        if (optionalLootBox.isEmpty()) {
            return ResponseEntity.notFound().build();
        }

        LootBoxEntity lootBox = optionalLootBox.get();
        RewardEntity reward = determineReward(lootBox);

        Optional<UserEntity> optionalUser = userService.getCurrentUser(token);
        if (optionalUser.isEmpty()) {
            return ResponseEntity.notFound().build();
        }

        UserEntity user = optionalUser.get();

        if (user.getBalance().compareTo(lootBox.getPrice()) < 0) {
            Map<String, String> errorResponse = new HashMap<>();
            errorResponse.put("error", "Insufficient balance");
            return ResponseEntity.badRequest().body(errorResponse);
        }

        user.setBalance(user.getBalance().subtract(lootBox.getPrice()));
        user.setBalance(user.getBalance().add(reward.getValue()));
        userRepository.save(user);

        return ResponseEntity.ok(determineReward(lootBox));
    }

    public RewardEntity determineReward(LootBoxEntity lootBox) {
        double randomValue = Math.random();
        BigDecimal cumulativeProbability = BigDecimal.ZERO;

        for (RewardEntity reward : lootBox.getRewards()) {
            cumulativeProbability = cumulativeProbability.add(reward.getProbability());
            if (randomValue <= cumulativeProbability.doubleValue()) {
                return reward;
            }
        }

        return lootBox.getRewards().getLast();
    }

}
